/*
	Copyright (c) 2004-2009, The Dojo Foundation All Rights Reserved.
	Available via Academic Free License >= 2.1 OR the modified BSD license.
	see: http://dojotoolkit.org/license for details
*/


if(!dojo._hasResource["fbfwca.Alerts"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["fbfwca.Alerts"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2009 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/
dojo.require("dojo._base.NodeList");
dojo.require("dijit.Toolbar");
dojo.require("dijit.form.DropDownButton");
dojo.require("dijit.Menu");
dojo.require("dijit._base.manager");
dojo.require("dijit.Tooltip");
dojo.require("ibm.tivoli.tip.dijit.TIPMessageDialog");
dojo.require("ibm.tivoli.tip.dijit.TIPButton");		
dojo.require("fbfwca.Grid");
dojo.require("fbfwca.Common");
dojo.require("fbfwca.ajax");
dojo.require("fbfwca.Localizable");
dojo.require("fbfwca.Formatters");
dojo.requireLocalization("nls", "general", null, "");
dojo.requireLocalization("nls_messages", "messages", null, "");
dojo.requireLocalization("dijit.form", "validate", null, "ROOT,ar,ca,cs,da,de,el,es,fi,fr,he,hu,it,ja,ko,nb,nl,pl,pt,pt-pt,ro,ru,sk,sl,sv,th,tr,zh,zh-tw");

dojo.provide("fbfwca.Alerts");

dojo.declare("fbfwca.Alerts", null, {

	/**************************************************************************
	* Constructor
	**************************************************************************/
	constructor : function() 
	{
		console.log("fbfwca.Alerts(): in constructor");
		this.mainPageId = 'MainTableHolder';
		this.EditAlertId = 'EditAlertForm';
		this.ToolsSettingId = 'ToolsSettingForm';
		this.connections = [];
		this.topics = [];
		this.gridObj = null;
		this.grid = null;
		this.folders = null;
		this.scripts = null;
		this.NLS = dojo.i18n.getLocalization("nls", "general");
		this.NLS_MESSAGES = dojo.i18n.getLocalization("nls_messages", "messages");

    // Get default alerts conditions
		this.alertsConditions = fbfwca.ajax.getJsonObject("alerts?action=types");
		console.log("alertsConditions=" + this.alertsConditions);
		
		this.stringConditions = [this.NLS.T_Equals, this.NLS.T_Not_equals, this.NLS.T_Less_than, this.NLS.T_Greater_than, this.NLS.T_In, this.NLS.T_Not_in];
		this.numberConditions = [this.NLS.T_Equals, this.NLS.T_Not_equals, this.NLS.T_Less_than, this.NLS.T_Greater_than];
      
    },

    /**************************************************************************
    * switch to alerts sub page, if necessary
    **************************************************************************/
    initPage : function (Folders, Scripts) 
    {
		this.folders = Folders;
		this.scripts = Scripts;
		this.InitAlertsTable();
  },

	/**************************************************************************
	* Cleanup function
	**************************************************************************/
	destroy : function () 
	{
	  	// Destroy all subscriptions
	  	dojo.forEach(this.topics, dojo.unsubscribe);
	  	delete this.topics;
	  	
	  	// Destroy all event connections
		dojo.forEach(this.connections, dojo.disconnect);
	  	delete this.connections;

		if (dijit.byId(this.EditAlertId)) {
			console.log("destroy EditAlertForm");
			dijit.byId(this.EditAlertId).destroyRecursive();
		}

		if (dijit.byId(this.ToolsSettingId)) {
			console.log("destroy ToolsSettingForm");
			dijit.byId(this.ToolsSettingId).destroyRecursive();
		}
			  	
		delete this.mainPageId;
		delete this.EditAlertId;
		delete this.ToolsSettingId;
		delete this.grid;
		delete this.gridObj;
		delete this.folders;
		delete this.scripts;
		delete this.alertsConditions;
		delete this.stringConditions;
		delete this.numberConditions;
		delete this.NLS;
		delete this.NLS_MESSAGES;
 	},
	  	
	/**************************************************************************
	* Creates & loads the alerts table.
	*
	* Parameter:
	*		none
	**************************************************************************/		
	InitAlertsTable : function () 
	{
    var textAlign;
    if(dojo.getComputedStyle(dojo.body()).direction == "rtl")
      textAlign = "text-align:right;";
    else
      textAlign = "text-align:left;";
      
		// Create the layout
		var layout = [
			{field: "_checked", name: "&nbsp;", width:"28px", editable:true, type:dojox.grid.cells.Bool,
					filter:false, styles: "text-align: center;" },
			{field: 'name', name: this.NLS.T_Name, width:'30%', cellClasses: "fauxLink", filter:true, styles:textAlign },
			{field: 'msgtext', name: this.NLS.T_Message_Text, width:'70%', filter:true, 
					formatter:fbfwca.Formatters.LongTextFormatter, styles:textAlign }
		];

		// Create toolbar widgets
		var	createButton = new ibm.tivoli.tip.dijit.TIPButton(
				{	label: this.NLS.T_Create_Alert_Conditions,
					showLabel: false, 
					iconClass: "newIcon" 
				});
		this.connections.push (dojo.connect(createButton, 'onClick', dojo.hitch(this, this.PerformAction, "create")));

		var	deleteButton = new ibm.tivoli.tip.dijit.TIPButton(
				{	label: this.NLS.T_Delete_Alert_Conditions,
					showLabel: false, 
					iconClass: "deleteIcon"
				});
		this.connections.push (dojo.connect(deleteButton, 'onClick', dojo.hitch(this, this.DeleteAlert)));

		var menu = new dijit.Menu();
		this.connections.push (dojo.connect(menu, 'onOpen', dojo.hitch(this, this.OnClick_TableAction)));
		
		var item = new dijit.MenuItem({
					 label: this.NLS.T_EMAIL_SCAN_SETTINGS,
					 title: this.NLS.T_EMAIL_SCAN_SETTINGS,
					 iconClass: "propertiesIcon",
					 id: "ToolsSettingID"
		});
		this.connections.push (dojo.connect(item, 'onClick', dojo.hitch(this, this.ConfigToolsSetting)));
		menu.addChild(item);

		item = new dijit.MenuItem({
				label: this.NLS.T_Create_Alert_Conditions,
				title: this.NLS.T_Create_Alert_Conditions,
				iconClass: "newIcon",
				id: "CreateAlertID"
		});
		this.connections.push (dojo.connect(item, 'onClick', dojo.hitch(this, this.PerformAction, "create")));
		menu.addChild(item);

		item = new dijit.MenuItem({
				label: this.NLS.T_Create_Like_Alert_Conditions,
				title: this.NLS.T_Create_Like_Alert_Conditions,
				iconClass: "newIcon",
				id: "CreateLikeAlertID",
				disabled: true
		});
		this.connections.push (dojo.connect(item, 'onClick', dojo.hitch(this, this.PerformAction, "createlike")));
		menu.addChild(item);
		
		item = new dijit.MenuItem({
				label: this.NLS.T_Modify_Alert_Conditions,
				title: this.NLS.T_Modify_Alert_Conditions,
         		iconClass: "editIcon",
				id: "ModifyAlertID",
				disabled: true
		});
		this.connections.push (dojo.connect(item, 'onClick', dojo.hitch(this, this.PerformAction, "modify")));
		menu.addChild(item);

		item = new dijit.MenuItem({
				label: this.NLS.T_Delete_Alert_Conditions,
				title: this.NLS.T_Delete_Alert_Conditions,
         		iconClass: "deleteIcon",
				id: "DeleteAlertID",
				disabled: true
		});
		this.connections.push (dojo.connect(item, 'onClick', dojo.hitch(this, this.DeleteAlert)));
		menu.addChild(item);

		//menu.addChild(new dijit.MenuSeparator());
                       
		var actionsDropDown = new dijit.form.DropDownButton({
	         label: this.NLS.T_ACTIONS,
	         title: this.NLS.T_ACTIONS,
	         dropDown: menu
		});
  
		// Create the grid
		var gridParms = { storeURL : "alerts",
							layout : layout,
							//refreshButton : false,
							filterOn : false,
							createButton : createButton,
							deleteButton : deleteButton,
							actionsDropDown : actionsDropDown
						};

		this.gridObj = new fbfwca.Grid(gridParms, document.createElement('div'));
		dojo.byId("alertsGridContainerNode").appendChild(this.gridObj.domNode);
		this.gridObj.resize();
		this.grid = this.gridObj.getTable();

		// Connect table default action
		this.connections.push (dojo.connect(this.grid, 'onCellClick', dojo.hitch(this, this.LaunchTableDefaultAction)));
		this.connections.push (dojo.connect(this.grid, "onCellMouseOver", this.ShowGridTooltip));
		this.connections.push (dojo.connect(this.grid, "onCellMouseOut", this.HideGridTooltip));
	},

	/**************************************************************************
	* Resize table object
	**************************************************************************/		
	ResizeTable : function (){
		if (this.gridObj)
			this.gridObj.resize();
	},
	
	/**************************************************************************
	* Hide tooltip
	*
	* Parameter:
	*		e - event
	**************************************************************************/		
	HideGridTooltip : function (e){
		dijit.hideTooltip(e.cellNode);
		dijit._masterTT._onDeck=null;
	},

	/**************************************************************************
	* Shows tooltip
	*
	* Parameter:
	*		e - event
	**************************************************************************/		
	ShowGridTooltip : function (e) {
		var row;
		if (e.grid.getCell(e.cellIndex).field == "msgtext")
		{
			row = e.grid.getItem(e.rowIndex);
			dijit.showTooltip(e.grid.store.getValue(row, "msgtext", "0"), e.cellNode);
		}
	},
	
	/**************************************************************************
	* Handles default table action.
	*
	* Parameter:
	*		e - event
	**************************************************************************/		
	LaunchTableDefaultAction : function (e) 
	{
		if (e.grid.getCell(e.cellIndex).field == "name")
		{
			this.PerformAction('modify');
			e.grid.selection.deselect(e.rowIndex);
		}
	},
	
	/**************************************************************************
	* Handles table action.
	*
	* Parameter:
	*		action - which action
	**************************************************************************/		
	PerformAction : function (action)
	{
		if (action == 'create' )
			this.ShowAlertForm('create');
		else if (action == 'createlike' )
			this.ShowAlertForm('createlike');
		else if (action == 'modify' )
			this.ShowAlertForm('modify');
		else
			alert("Invalid action");
	},

	/**************************************************************************
	* Initializes and shows the Configure Tools Setting form.
	*
	* Parameter:
	*		none
	**************************************************************************/		
	ConfigToolsSetting : function ()
	{
		console.log("ConfigToolsSetting:");
		dijit.byId("ToolsSettingFormOK").attr("label", this.NLS.T_OK);
		dijit.byId("ToolsSettingFormCancel").attr("label", this.NLS.T_CANCEL);

		dijit.byId("ToolsSettingFormOK").attr("title", this.NLS.T_OK);
		dijit.byId("ToolsSettingFormCancel").attr("title", this.NLS.T_CANCEL);
				
		var data = fbfwca.ajax.getMailServer();
		var settings = this.GetServerScanSetting();
		dijit.byId('Scan_frequency').attr('value', settings.value);
		dijit.byId('Scan_frequency_unitFS').attr('value', settings.unit);
	  delete settings;
    
		if (data)
		{
			console.log("ConfigToolsSetting: restore from server");
			dijit.byId('SMTP_address').attr("value", data.mail_server);
			dijit.byId('SMTP_port').attr("value", data.port);
			dijit.byId('Username').attr("value", data.username);
			dijit.byId('UserPwd').attr("value", data.password);
			dijit.byId('UserVerifyPwd').attr("value", data.password);
		}
   	else
		{
			console.log("ConfigToolsSetting: restore from default");
			this.ResetToolForm();
		}
  
		fbfwca.Common.ShowForm(this.ToolsSettingId, this.mainPageId);
	},

	/**************************************************************************
	* Gets scan frequency setting from the server.
	*
	* Parameter:
	*		none
	* Return:
	*		an object 
	*				settings.value
	*				settings.unit
	**************************************************************************/		
	GetServerScanSetting : function ()
	{
		var obj = fbfwca.ajax.getSettings();
		var settings;
		if ( obj )
			settings = fbfwca.Formatters.ConvertTimeValue(obj.new_client_scan_interval);
		else
		{
			settings = new Object();
			settings.value = 2;
			settings.unit = 0;
		}
		
		return settings;
	},
		
	/**************************************************************************
	* Initializes and shows the Edit Alert form.
	*
	* Parameter:
	*		action - which action
	**************************************************************************/		
	ShowAlertForm : function (action)
	{
		dijit.byId("AlertOKBtn").attr("label", this.NLS.T_OK);
		dijit.byId("AlertAnotherBtn").attr("label", this.NLS.T_ADD_ANOTHER);
		dijit.byId("AlertCancelBtn").attr("label", this.NLS.T_CANCEL);
		dijit.byId("AlertOKBtn2").attr("label", this.NLS.T_OK);
		dijit.byId("AlertCancelBtn2").attr("label", this.NLS.T_CANCEL);
		dijit.byId("EditAlertFormBrowse").attr("label", this.NLS.T_BROWSE);

		dijit.byId("AlertOKBtn").attr("title", this.NLS.T_OK);
		dijit.byId("AlertAnotherBtn").attr("title", this.NLS.T_ADD_ANOTHER);
		dijit.byId("AlertCancelBtn").attr("title", this.NLS.T_CANCEL);
		dijit.byId("AlertOKBtn2").attr("title", this.NLS.T_OK);
		dijit.byId("AlertCancelBtn2").attr("title", this.NLS.T_CANCEL);
		dijit.byId("EditAlertFormBrowse").attr("title", this.NLS.T_BROWSE);
				
		var div1 = document.getElementById("EditAlertFormHead");
		var name = dijit.byId('Alert_name');
		var alertData = null;
		var i;

		if (action == "create" || action == "createlike")
		{
			div1.innerHTML = this.NLS.T_Create_Alert_Conditions;
			dojo.style('Alert_ThreeButtons', 'display', 'block');
			dojo.style('Alert_TwoButtons', 'display', 'none');
			name.attr('disabled', false);
			name.promptMessage = this.NLS.T_REQUIRED_FIELD;
			this.ResetAlertForm();
      		//name.focus();
    	}
		else
		{
			div1.innerHTML = this.NLS.T_Modify_Alert_Conditions;
			dojo.style('Alert_TwoButtons', 'display', 'block');
			dojo.style('Alert_ThreeButtons', 'display', 'none');
			name.attr('disabled', true);
			//dijit.byId("Alert_msgtext").focus();
		}		
		
		if ( action == "createlike" || action == "modify")
		{
				var item = this.grid.selection.getFirstSelected();
				var alertname = this.grid.store.getValue(item, "name");
				if ( action == "modify" )
					name.attr('value', alertname);
			
				alertData = fbfwca.ajax.getAlert(alertname);
		}
  
    // fill Rules container
    if (alertData)
    {
			console.log("ShowAlertForm: restore from server");
			dijit.byId('Alert_msgtext').attr("value", alertData.msg);
			var addresses = alertData.addresses.join(";");
			dijit.byId('Alert_email').attr("value", addresses);
			
			if (alertData.action == 0)
			{
      	dijit.byId('Normal_rb').attr('checked', true);
				dijit.byId('Warning_rb').attr('checked', false);
				dijit.byId('Critical_rb').attr('checked', false);
			}
			else if (alertData.action == 1)
    	{
      	dijit.byId('Normal_rb').attr('checked', false);
				dijit.byId('Warning_rb').attr('checked', true);
				dijit.byId('Critical_rb').attr('checked', false);
			}
			else
 			{
 				dijit.byId('Normal_rb').attr('checked', false);
				dijit.byId('Warning_rb').attr('checked', false);
				dijit.byId('Critical_rb').attr('checked', true);
			}	
		
			if (alertData.mode == 0)	
			{			
				dijit.byId('AlertAND_rb').attr('checked', true);
				dijit.byId('AlertOR_rb').attr('checked', false);
   		}
			else
			{
				dijit.byId('AlertAND_rb').attr('checked', false);
				dijit.byId('AlertOR_rb').attr('checked', true);
   		}

			for (i=0; i<alertData.conditions.length; i++)
			{
				console.log("addRules " + alertData.conditions[i].name + " " + alertData.conditions[i].relation + " " + alertData.conditions[i].value);
				this.AddRules(alertData.conditions[i].name, alertData.conditions[i].relation, alertData.conditions[i].value);
			}
		}
		else
			var baseNode = this.AddRules(null, 0, null);

		fbfwca.Common.ShowForm(this.EditAlertId, this.mainPageId);
	},

	/**************************************************************************
	* Resets widgets on the Edit Alert form.
	* Mainly to initialize fields to default value.
	*
	* Parameter:
	*		none
	**************************************************************************/		
	ResetAlertForm : function ()
	{
		var name = dijit.byId('Alert_name');
		name.attr('value', "");
		name.reset();
		
		dijit.byId('Alert_msgtext').attr('value', "");
   		dijit.byId('Alert_email').attr('value', "");

		dijit.byId('Normal_rb').attr('checked', "checked");
		dijit.byId('AlertAND_rb').attr('checked', "checked");

		//add any other
	},

	/**************************************************************************
	* Issues a confirm message for the deletion.
	*
	* Parameter:
	*		none
	**************************************************************************/		
	DeleteAlert : function () 
	{
		var msgDlg;
		if ( this.grid.selection.getSelectedCount() == 0 )
		{
				msgDlg = new ibm.tivoli.tip.dijit.TIPMessageDialog(
						{message: this.NLS_MESSAGES.FBW1002E,
						 messageId: "FBW1002E",
						 type: "Error"});
		}
		else
		{
				var buttons = [this.NLS.T_OK, this.NLS.T_CANCEL];
				msgDlg = new ibm.tivoli.tip.dijit.TIPMessageDialog(
						{message: this.NLS_MESSAGES.FBW1001C,
						 messageId: "FBW1001C",
						 type: "Confirm",
						 callback: this.DeleteAlertCallback,
						 callbackParms: {grid: this.grid, gridObj: this.gridObj},
						 buttons: buttons});
		}
		msgDlg.show();
    },

	/**************************************************************************
	* Delete call back.  Called by TIPMessageDialog.
	*
	* Parameter:
	*		buttonPressedId - button pressed
	*		messageId - message ID
	*		checked - show checked box check
	*		parms - {grid: this.grid, gridObj: this.gridObj}
	**************************************************************************/		
	DeleteAlertCallback : function (buttonPressedId, messageId, checked, parms) 
	{		
		if (buttonPressedId == 0)
		{
		  var count = 0;
	    parms.grid.store.fetch(
		  {
			  query: {_checked:true},
			  onItem: function(item, request) {
					  console.log("DeleteAlertCallback: " + item.name);
					  fbfwca.ajax.deleteAlert(item.name);
					  parms.grid.store.deleteItem(item);
					  count++;
			  }
		  });
		  parms.grid.store.save();
		  parms.grid.sort();
		  parms.gridObj.updateFooter(parms.grid.rowCount-count, 0);
		}
	},

	/**************************************************************************
	* Handles button pressed on Edit Alert form.
	*
	* Parameter:
	*		buttonId - which button was pressed (0=OK, 1=Add Another, 2=Cancel)
	**************************************************************************/		
	OnClick_EditAlert : function (buttonId) 
	{
		// Cancel button
		if ( buttonId == 2 )
		{
			this.CleanupAlertForm();
			fbfwca.Common.HideForm(this.EditAlertId, this.mainPageId);
			this.RefreshTables();  
			return;
		}

		// OK or add another button
		var name = dijit.byId('Alert_name');
		
		// Make sure required fields were entered.
		if (name.attr('disabled') == false && !name.validate(true) )
				return;
		
		var alert = new Object();
		alert.name = name.attr('value');
	    alert.msg = dijit.byId('Alert_msgtext').attr('value');
	    alert.addresses = new Array();
	    var emails = dijit.byId('Alert_email').attr('value').split(";");
	    var i, idx = 0;
		for (i=0; i<emails.length; i++)
		{
			if ( emails[i].length > 0 )
			{
				alert.addresses[idx] = "" + this.trim(emails[i]);
				idx++;
			}
		}
		if ( dijit.byId('Normal_rb').attr('checked') )
			alert.action = 0;
		else if ( dijit.byId('Warning_rb').attr('checked') )
			alert.action = 1;
		else
			alert.action = 2;

		if ( dijit.byId('AlertAND_rb').attr('checked') )
			alert.mode = 0;
		else
			alert.mode = 1;
			
		alert.conditions = new Array();
		var selectNodes = dojo.query("div > select", 'RulesContainer');
		var inputNodes = dojo.query("div > input", 'RulesContainer');
		var num = 0;
		for (i=0; i<inputNodes.length;i++)
		{
			if (i!=1 && i!=2)
			{
				alert.conditions[num] = new Object();
				var index = selectNodes[num*2].selectedIndex;
				alert.conditions[num].name = "" + selectNodes[num*2].options[index].text;
				alert.conditions[num].relation = selectNodes[(num*2)+1].selectedIndex;
				alert.conditions[num].value = "" + inputNodes[i].value;
				if ( !this.validateInput(inputNodes[i], alert.conditions[num].name, alert.conditions[num].value) )
					return;
				num++;
			}
		}
 
		// Create action 
		var response;
		if (name.attr('disabled') == false)
			response = fbfwca.ajax.createAlert(alert);
		else
			response = fbfwca.ajax.modifyAlert(alert);
		
		delete alert.addresses;
		delete alert.conditions;
		delete alert;
		
		if ( response == -1 )
			return;
				
		// Add another case
		if (buttonId == 1)
		{
			this.CleanupAlertForm();
			this.AddRules(null, 0, null);
			this.ResetAlertForm();
		}
		else
	    {
			this.CleanupAlertForm();
			fbfwca.Common.HideForm(this.EditAlertId, this.mainPageId);
			this.RefreshTables();
	    }
	},

	RefreshTables : function () 
	{
		this.gridObj.onRefresh();
		this.folders.gridObj.onRefresh();
		this.scripts.gridObj.onRefresh();
	},
	
	/**************************************************************************
	* Verify long input string contains a number.
	*
	* Parameter:
	*		node - input dom node
	*		condition - alert conditions string
	*		value - a string value
	**************************************************************************/		
	validateInput : function (node, condition, value)
	{
		var dojoMsgs = dojo.i18n.getLocalization("dijit.form", "validate");

		console.log("validateInput: condition=" + condition + " " + value);
		
		if ( value.length == 0 )
		{
			dijit.showTooltip(this.NLS.T_REQUIRED_FIELD, node);
			return false;
		}
			
		var whichType = this.GetConditionsType(condition);
		if ( whichType == "long" )
		{
			var val = parseInt(value);
			if (isNaN(value))
			{
				dijit.showTooltip(dojoMsgs.invalidMessage, node);
				return false;
			}
			
		}
	
		dijit.hideTooltip(node);
		return true;
	},
	
	/**************************************************************************
	* Removes space on the left and right of string
	*
	* Parameter:
	*		s - a string
	**************************************************************************/		
	trim : function (s)
	{
		var l=0;
		var r=s.length-1;
		while (l < s.length && s[l] == ' ')
			l++;
		while (r > 1 && s[r] == ' ')
			r-=1;
		
		return s.substring(l,r+1);
	},
	
	/**************************************************************************
	* Handles buttons on Tools Setting form.
	*
	* Parameter:
	*		buttonId - which button was pressed (0=OK, 1=Cancel)
	**************************************************************************/		
	OnClick_ToolsSettingForm : function (buttonId) 
	{
    	console.log("OnClick_ToolsSettingForm:");

		// Cancel button
		if ( buttonId == 1 )
		{
			fbfwca.Common.HideForm(this.ToolsSettingId, this.mainPageId);
			this.RefreshTables();
			return;
		}
		
		if (dijit.byId('UserPwd').attr("value") != dijit.byId('UserVerifyPwd').attr("value"))
		{
			var msgDlg;
			msgDlg = new ibm.tivoli.tip.dijit.TIPMessageDialog(
					{message: this.NLS_MESSAGES.FBW1005E,
					 messageId: "FBW1005E",
					 type: "Error"});

			msgDlg.show();
			return;
		}

		// OK button
		var obj = new Object();
		obj.new_client_scan_interval = fbfwca.Formatters.GetTimeValue(dijit.byId('Scan_frequency').attr("value"),
																	dijit.byId('Scan_frequency_unitFS').attr("value"));
		var response = fbfwca.ajax.setSettings(obj);
		delete obj;
		if ( response == -1 )
			return;
					
		response = fbfwca.ajax.postMailServer(dijit.byId('SMTP_address').attr("value"), dijit.byId('Username').attr("value"),
					dijit.byId('UserPwd').attr("value"), dijit.byId('SMTP_port').attr("value"));
		if ( response == -1 )
			return;
			
		fbfwca.Common.HideForm(this.ToolsSettingId, this.mainPageId);
		
		this.RefreshTables();
	},

	/**************************************************************************
	* Resets widgets on the Tools Setting form.
	*
	* Parameter:
	*		none
	**************************************************************************/		
	ResetToolForm : function ()
	{
	    //dijit.byId("Scan_frequency").attr("value",2);
	    //dijit.byId("Scan_frequency_unitFS").attr('value', '0');
	    dijit.byId('SMTP_address').attr("value","");
	    dijit.byId('SMTP_port').attr("value","");
	    dijit.byId('Username').attr("value","");
	    dijit.byId('UserPwd').attr("value","");
	    dijit.byId('UserVerifyPwd').attr("value","");
	},
		
	/**************************************************************************
	* Enables or disables actions based on user's action selection.
	*
	* Parameter:
	*		none
	**************************************************************************/		
	OnClick_TableAction : function () 
	{
		 var count = this.grid.selection.getSelectedCount();
		
		 var CreateAction = dijit.byId("CreateLikeAlertID");
		 var DeleteAction = dijit.byId("DeleteAlertID");
		 var ModifyAction = dijit.byId("ModifyAlertID");

		 if ( count == 0 )
		 {
			CreateAction.attr('disabled', true);
			DeleteAction.attr('disabled', true);
			ModifyAction.attr('disabled', true);
		 }
		 else if ( count == 1 )
		 {
			CreateAction.attr('disabled', false);
			DeleteAction.attr('disabled', false);
			ModifyAction.attr('disabled', false);
		 }
		 else
		 {
			CreateAction.attr('disabled', true);
			DeleteAction.attr('disabled', false);
			ModifyAction.attr('disabled', true);
		 }
	},
     
	/**************************************************************************
	* Performs cleanup for the alert form.
	* Important to reset any required widgets & hide tooltip
	*
	* Parameter:
	*		none
	**************************************************************************/		
	CleanupAlertForm : function () 
	{
		var widget = dijit.byId('Alert_name');
		if (widget.attr('disabled') == false)
		{
			dijit.hideTooltip(widget.domNode);
			widget.reset();
		}

		var nodes = dojo.query("div > input", 'RulesContainer');
		if (nodes.length > 0) {
		    dojo.forEach(nodes, function(node) {
		    	console.log("id=" + node.id);
		        dijit.hideTooltip(node);
		    }); // end forEach
		}
		dojo.query("#AlertRulesContentPane", "RulesContainer").orphan();
	},
		
	/**************************************************************************
	* Shows email address dialog and accept user's selection.
	*
	* Parameter:
	*		none
	**************************************************************************/	
    selectEmailAddr : function () {
       
		// Create the layout
		var  dialogId = dijit.byId("emailAddrDialog");
		dialogId.titleNode.innerHTML = this.NLS.T_Alert_Recipients;
		//console.log("show dialog with id: "+ dialogId);
		dijit.byId("EmailaddrDialogOK").attr("label", this.NLS.T_OK);
		dijit.byId("EmailaddrDialogCancel").attr("label", this.NLS.T_CANCEL);

		dijit.byId("EmailaddrDialogOK").attr("title", this.NLS.T_OK);
		dijit.byId("EmailaddrDialogCancel").attr("title", this.NLS.T_CANCEL);

		dialogId.show();       

    var textAlign;
    if(dojo.getComputedStyle(dojo.body()).direction == "rtl")
      textAlign = "text-align:right;";
    else
      textAlign = "text-align:left;";
      
    var layout = [
        {field: "_checked", name: "&nbsp;", width:"28px", editable:true, type:dojox.grid.cells.Bool,
        		filter:false, styles: "text-align: center;" },
        {field: 'address', name: this.NLS.T_Email_Address, width:'100%', filter:true, styles:textAlign }
    ];

    // Create the grid
		var store = fbfwca.ajax.getMailAddresses();
        var gridParms = {store : store,
                         layout : layout,
                         refreshButton : false
                        };

        this.AddrGridObj = new fbfwca.Grid(gridParms, document.createElement('div'));
        dojo.byId("emailaddrGridContainerNode").appendChild(this.AddrGridObj.domNode);
        this.AddrGridObj.resize();
        this.AddrGrid = this.AddrGridObj.getTable();

    },

    /**************************************************************************
    * Handles buttons on Email Addr. Selection form.
    *
    * Parameter:
    *		buttonId - which button was pressed (0=OK, 1=Cancel)
    **************************************************************************/		
    OnClick_EmailSelectForm : function (buttonId) {
        console.log("dialog button clicked!"); 
        var selectedRowList = "";

        // OK button
        if (buttonId == 0)
        {
            var selectionArray = this.AddrGrid.selection.getSelected();
            
            var row= null;
            var rowStr = null;
            dojo.forEach(selectionArray, function(row) 
            {
                if (row != null) 
                {
					rowStr = row.address;
					if ( selectedRowList.length == 0 )
						selectedRowList = selectedRowList + rowStr;
					else
						selectedRowList = selectedRowList + ";" + rowStr;
				}
			});
            console.log("current selection: "+selectedRowList);
            dijit.byId("Alert_email").attr('value', selectedRowList);
        }

        if (this.AddrGridObj){
            this.AddrGridObj.destroy();
        }

        dijit.byId("emailAddrDialog").hide();
        return;
    },
    
    /**************************************************************************
    * Handles REMOVE RULES button.
    *
    * Parameter:
    *		none
    **************************************************************************/		
    RemoveRules : function () 
    {
        console.log("REMOVE button clicked!");
        var nodes = dojo.query("div > select", 'RulesContainer');

        if(nodes.length >2)
        {
			if (dojo.isIE)
			{						
				var spanlist = dojo.query("div > span", 'RulesContainer'); 
				var id;
				var node;
			
				
				/* Delete span node */
				id = "" + "#" + spanlist[spanlist.length-1].id;
				console.log("span_id1=" + id);
				node = dojo.query(id).orphan();
	
				/* Delete input node */
				var inputlist = dojo.query("div > input", 'RulesContainer'); 
				id = "" + "#" + inputlist[inputlist.length-1].id;
				console.log("input_id1=" + id);
				dijit.hideTooltip(inputlist[inputlist.length-1]);
				node = dojo.query(id).orphan();
	
				/* Delete span node */
				id = "" + "#" + spanlist[spanlist.length-2].id;
				console.log("span_id2=" + id);
				node = dojo.query(id).orphan();
	
				/* Delete select node */
				id = "" + "#" + nodes[nodes.length-1].id;
				console.log("select_id2=" + id);
				node = dojo.query(id).orphan();
	
				/* Delete span node */
				id = "" + "#" + spanlist[spanlist.length-3].id;
				console.log("span_id3=" + id);
				node = dojo.query(id).orphan();
	
				/* Delete select node */
				id = "" + "#" + nodes[nodes.length-2].id;
				console.log("select_id1=" + id);
				node = dojo.query(id).orphan();
			}
			else
			{
				var inputlist = dojo.query("div > input", 'RulesContainer');
				dijit.hideTooltip(inputlist[inputlist.length-1]);
				dojo.query("div > span:last-child", 'RulesContainer').orphan();
				dojo.query("div > input:last-child", 'RulesContainer').orphan();
				dojo.query("div > span:last-child", 'RulesContainer').orphan();
				dojo.query("div > select:last-child", 'RulesContainer').orphan();
				dojo.query("div > span:last-child", 'RulesContainer').orphan();
				dojo.query("div > select:last-child", 'RulesContainer').orphan();
			}
        }
    },
            
    /**************************************************************************
    * Handles ADD RULES button.
    *
    * Parameter:
    *		alertName - string value of the first filtering select (null or string)
    *		relation - index value of the second filtering select (a number)
    *		value- string value of the input field (null or string)
    **************************************************************************/		
    AddRules : function (alertName, relation, value)
    {	
        console.log("AddRules: " + alertName + " " +  relation + " " + value );
        var output = "";
				var baseBorderContainerNode = dojo.query("#RulesContainer");
        var baseNode = dojo.query("#AlertRulesContentPane","RulesContainer");

		/* Check if we have a container first */
		if ( baseNode.length > 0 )
			console.log("AddRules: container exist");
		else
		{
			console.log("AddRules: no container");
      var d = document.getElementById("RulesContainer");
      if (d.style.width=="700px")
        baseBorderContainerNode.addContent('<div id="AlertRulesContentPane" dojoType="dijit.layout.ContentPane" region="center" style="overflow:auto;width:689px;height:72px;padding:10px 0px 10px 10px;"/>');
      else
        baseBorderContainerNode.addContent('<div id="AlertRulesContentPane" dojoType="dijit.layout.ContentPane" region="center" style="overflow:auto;width:789px;height:72px;padding:10px 0px 10px 10px;"/>');

			baseNode = dojo.query("#AlertRulesContentPane","RulesContainer");
		}

        var nodeList = dojo.query("#AlertRulesContentPane > ","RulesContainer");
        console.log("nodeList.length=" + nodeList.length);
        
        /* Generate initial name and append node number to create uniqueness */
        var selectionName1 = 'RuleSelection' + nodeList.length+1;
        
        /* Add 1st filteringSelect node */
        output = this.CreateConditionSelection(alertName, selectionName1);
        baseNode.addContent(output,'last');

/*
        var select = document.createElement("select");
        select.id = selectionName1;
        select.onchange = function(){alerts.LoadConditions(this);};
        var i;
		for (i=0; i< this.alertsConditions.items.length; i++)
		{
			var option = document.createElement("option");
			var text = document.createTextNode(this.alertsConditions.items[i].name);
			option.appendChild(text);
			if (alertName && (alertName == this.alertsConditions.items[i].name))
				option.selected = true;
			select.appendChild(option);
		}
        baseNode.addContent(select,'last');
*/
        /* Add 1st span node */
        var spanContent = new Array();
        spanContent[0] = '<span class="tab" id="';
        spanContent[1] = selectionName1 + 'span"></span>';
        output = spanContent.join(""); 
        baseNode.addContent(output,'last');
        
        /* Add 2nd filteringSelect node */
        output = this.CreateRelationSelection(alertName, relation, selectionName1, true);
        nodes = baseNode.addContent(output,'last');

        /* Add 2nd span node */
        spanContent[1] = selectionName1 + 'span_2"></span>';
        output = spanContent.join("");
        baseNode.addContent(output,'last');
        
        /* Add 1st input node */
        var input = document.createElement("input");
        input.type = "text";
        input.id = selectionName1+"input";
        input.style.width="180px";
        input.onfocus = function(){dijit.hideTooltip(this);};
        if ( value )
        	input.value=value;
        baseNode.addContent(input,'last');
        
        /* Add 3rd span node */
        spanContent[1] = selectionName1 + 'span_3"></span>';
        output = spanContent.join("");
        baseNode.addContent(output,'last');
       
		if ( nodeList.length == 0 )
		{
			/* Add image button nodes */
			var img = document.createElement('input');
			img.type = "image";
			img.src = fbfwca_root + "/jsp/CSS/images/ac18_remove_default_24.png";
			img.id = selectionName1 + "img";
			img.title = this.NLS.T_Remove_Condition;
			img.alt = this.NLS.T_Remove_Condition;
			img.onmouseover = function (){this.src=fbfwca_root + "/jsp/CSS/images/ac18_remove_over_24.png";};
			img.onmouseout = function (){this.src=fbfwca_root + "/jsp/CSS/images/ac18_remove_default_24.png";};
			img.onclick = this.RemoveRules;
			baseNode.addContent(img,'last');

			var img2 = document.createElement('input');
			img2.src = fbfwca_root + "/jsp/CSS/images/ac18_add_default_24.png";
			img2.type = "image";
			img2.id = selectionName1 + "img_2";
			img2.title = this.NLS.T_Add_Condition;
			img2.alt = this.NLS.T_Add_Condition;
			img2.onmouseover = function (){this.src=fbfwca_root + "/jsp/CSS/images/ac18_add_over_24.png";};
			img2.onmouseout = function (){this.src=fbfwca_root + "/jsp/CSS/images/ac18_add_default_24.png";};
			img2.onclick = function () {alerts.AddRules(null, 0, null);};
			baseNode.addContent(img2,'last');
		}
				
        return baseNode;                                                      
    },

    /**************************************************************************
    * Constructs the 1st filtering select.
    *
    * Parameter:
    *		alertName - string value of the first filtering select (null or string)
    *		selectionName1 - ID string value of the condition FS (1st filtering select)
    **************************************************************************/		
	CreateConditionSelection : function (alertName, selectionName1) 
	{
		/* Add 2nd filteringSelect node */
		var output = "";
		var i;

		output += '<select dojoType="dijit.form.FilteringSelect" id="' + selectionName1 + '" onchange="alerts.LoadConditions(this);">';

		for (i=0; i< this.alertsConditions.items.length; i++)
		{
			if (alertName && (alertName == this.alertsConditions.items[i].name))
					output += '<option selected>' + this.alertsConditions.items[i].name + '</option>';
			else
					output += '<option>' + this.alertsConditions.items[i].name + '</option>';
		}

		output += '</select>';
		
		console.log("CreateConditionSelection: " + output);

		return output;
	},
	
    /**************************************************************************
    * Constructs the 2nd filtering select.
    *
    * Parameter:
    *		alertName - string value of the first filtering select (null or string)
    *		relation - index value of the second filtering select (a number)
    *		selectionName1 - ID string value of the condition FS (1st filtering select)
    *		createSelect - true=create select html, false=create options html
    **************************************************************************/		
	CreateRelationSelection : function (alertName, relation, selectionName1, createSelect) 
	{
		/* Add 2nd filteringSelect node */
		var output = "";
		var i;
		
		if ( createSelect )
		{
			output += '<select dojoType="dijit.form.FilteringSelect" id="';
			output += selectionName1 + '_2">';
		}
     
		var whichType = this.GetConditionsType(alertName ? alertName : this.alertsConditions.items[0].name);
		if ( whichType == "string" )
		{
			for (i=0; i< this.stringConditions.length; i++)
			{
				if (i == relation)
					output += '<option selected>' + this.stringConditions[i] + '</option>';
				else
					output += '<option>' + this.stringConditions[i] + '</option>';
			}
		}
		else
		{
			for (i=0; i< this.numberConditions.length; i++)
			{
				if (relation == i)
					output += '<option selected>' + this.numberConditions[i] + '</option>';
				else
					output += '<option>' + this.numberConditions[i] + '</option>';
			}
		}

		if ( createSelect )
			output += '</select>';
		
		console.log("CreateRelationSelection: " + output);

		return output;
	},
			
    /**************************************************************************
    * Handles onChange of the alert rules filtering select.
    *
    * Parameter:
    *		widget - html node of the filtering select receiving an onchange event
    **************************************************************************/		
    LoadConditions : function  (widget) 
    {
    //var id = widget.id + "_2";
    //var d = dojo.byId(id);
    //var t = dijit.byId(id);
		console.log ("LoadConditions: " + dojo.byId(widget.id + "_2").innerHTML);

		var index = widget.selectedIndex;
		var alertName = widget.options[index].text;
		console.log("index=" + index + " alertName=" + alertName);
		
		if (dojo.isIE)
		{
			var output = this.CreateRelationSelection(alertName, 0, widget.id, true);
			dojo.byId(widget.id + "_2").outerHTML = output;
			console.log("IE selections=" + dojo.byId(widget.id + "_2").outerHTML);
		}
		else
		{
			var output = this.CreateRelationSelection(alertName, 0, widget.id, false);
			dojo.byId(widget.id + "_2").innerHTML = output;
			console.log("FF selections=" + dojo.byId(widget.id + "_2").innerHTML);
		}

	}, 
		
    /**************************************************************************
    * Based on the pre-set alert name, returns its condition type.
    * Either a value "string" or "long".
    * 
    * Parameter:
    *		alertName - string value
    **************************************************************************/		
	GetConditionsType : function (alertName) 
	{
		console.log("GetConditionsType: " + alertName);
		var i;
		var type;
		for (i=0; i< this.alertsConditions.items.length; i++)
		{
			if (alertName == this.alertsConditions.items[i].name)
				type = "" + this.alertsConditions.items[i].type;
		}
		
		console.log("GetConditionsType: type=" + type);
		return type;
	}
});

}
