/*
	Copyright (c) 2004-2009, The Dojo Foundation All Rights Reserved.
	Available via Academic Free License >= 2.1 OR the modified BSD license.
	see: http://dojotoolkit.org/license for details
*/


if(!dojo._hasResource["fbfwca.Formatters"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["fbfwca.Formatters"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2009 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/
dojo.provide("fbfwca.Formatters");
dojo.require("dojo.number");
dojo.require("dojo.date.locale");
dojo.requireLocalization("nls", "general", null, "", "");

fbfwca.Formatters = {
	
	/**************************************************************************
	* Converts storage value in bytes to appropriate unit in a string format.
	* Example:  20 GB, 20 MB
	*
	* Parameter:
	*		value - storage value
	**************************************************************************/		
	StorageValueFormatter: function (value) {
		var NLS = dojo.i18n.getLocalization("nls", "general");	

		// Given a value that represents bytes, return a string that repsents the value, rounded to the nearest storage unit:
    	var multipliers = [1024*1024*1024*1024*1024, 1024*1024*1024*1024, 1024*1024*1024, 1024*1024, 1024, 1];
		var suffixes = [NLS.T_PB, NLS.T_TB, NLS.T_GB, NLS.T_MB, NLS.T_KB, NLS.T_bytes];
		var val = parseInt(value, 10);
		var newValue = "0";
		var suffix = suffixes[5];

    	for (i=0; i < multipliers.length-1; i++) 
    	{
        	if (val >= multipliers[i]) 
        	{
	            newValue = val / multipliers[i];
	            suffix = suffixes[i];
	            break;
        	}
    	}

		return Math.round(newValue) + " " + suffix;
	},

	/**************************************************************************
	* Converts storage value in bytes to appropriate unit in an array format
	* where the 1st value is the number and the 2nd value is the index of the
	* storage unit.
	* Example: ["20",2] means 20GB
	*
	* Parameter:
	*		value - storage value
	**************************************************************************/		
	ConvertStorageValue: function (value) {

		// Given a value that represents bytes, return a string that repsents the value, rounded to the nearest storage unit:
	    var multipliers = [1024*1024*1024*1024*1024, 1024*1024*1024*1024, 1024*1024*1024, 1024*1024, 1024, 1];
			var suffixes = [4, 3, 2, 1, 0, 0];
			var val = parseInt(value, 10);
			var newValue = "0";
			var suffix = 0;
	
	    for (i=0; i < multipliers.length-1; i++) {
	        if (val >= multipliers[i]) {
	            newValue = val / multipliers[i];
	            suffix = suffixes[i];
	            break;
	        }
	    }
    
	    var out = new Object();
	    out.value = Math.round(newValue).toString();
	    out.unit = suffix;
		return out;
	},

	/**************************************************************************
	* Returns the storage unit value based on the input index value.
	*
	* Parameter:
	*		index - the filteringselect index
	**************************************************************************/		
	GetStorageUnit: function (index) {
		var NLS = dojo.i18n.getLocalization("nls", "general");	
		var suffixes = [NLS.T_KB, NLS.T_MB, NLS.T_GB, NLS.T_TB, NLS.T_PB];
		return suffixes[index];
	},

	/**************************************************************************
	* Converts storage value to bytes.
	*
	* Parameter:
	*		value - storage value
	*		unit - storage value unit
	**************************************************************************/		
	ConvertStorageValueToBytes: function (value, unit) {
		var multipliers = [1024, 1024*1024, 1024*1024*1024, 1024*1024*1024*1024, 1024*1024*1024*1024*1024];
		return (value * multipliers[unit]);
	},
		
	/**************************************************************************
	* Converts throttle value in kilobits to appropriate unit in a string format.
	* Example:  20 Gbs, 20 Mbs
	*
	* Parameter:
	*		value - throttle value in kilobits per second
	**************************************************************************/		
	ThrottleValueFormatter: function (value) {
		var NLS = dojo.i18n.getLocalization("nls", "general");	
	    var multipliers = [1024*1024, 1024, 1];
		var suffixes = [NLS.T_Gbs, NLS.T_Mbs, NLS.T_Kbs];
		var val = parseInt(value, 10);
		var newValue = "0";
		var suffix = suffixes[2];
	
	    for (i=0; i < multipliers.length; i++) {
	        if (val >= multipliers[i]) {
	            newValue = val / multipliers[i];
	            suffix = suffixes[i];
	            break;
	        }
	    }
    
		return Math.round(newValue) +" "+suffix;
	},

	/**************************************************************************
	* Converts throttle value in kilobits to appropriate unit in an array format
	* where the 1st value is the number and the 2nd value is the index of the
	* throttle unit.
	* Example: [20,1] means 20Mbs
	*
	* Parameter:
	*		value - storage value
	**************************************************************************/		
	ConvertThrottleValue: function (value) {
	    var multipliers = [1024*1024, 1024, 1];
		var suffixes = [2, 1, 0];
		var val = parseInt(value, 10);
		var newValue = "0";
		var suffix = 0;
	
	    for (i=0; i < multipliers.length; i++) {
	        if (val >= multipliers[i]) {
	            newValue = val / multipliers[i];
	            suffix = suffixes[i];
	            break;
	        }
	    }
    
	    var out = new Object();
	    out.value = Math.round(newValue).toString();
	    out.unit = suffix;
		return out;
	},

	/**************************************************************************
	* Converts throttle value to kilobits
	*
	* Parameter:
	*		value - throttle value
	*		unit - throttle value unit
	**************************************************************************/		
	ConvertThrottleValueToKilobits: function (value, unit) {
    	var multipliers = [1, 1024, 1024*1024];
		return (Math.round(value * multipliers[unit]));
	},
	
	/**************************************************************************
	* Returns the throttle unit value based on the input index value.
	*
	* Parameter:
	*		index - the filteringselect index
	**************************************************************************/		
	GetThrottleUnit: function (index) {
		var NLS = dojo.i18n.getLocalization("nls", "general");	
		var suffixes = [NLS.T_Kbs, NLS.T_Mbs, NLS.T_Gbs];
		return suffixes[index];
	},

	/**************************************************************************
	* Converts time value in milliseconds to appropriate unit and return in
	* an object format:
	*    time.value = 2 (value of 2)
	*    time.unit = 0 (index for minutes)
	*
	* Parameter:
	*		value - storage value
	**************************************************************************/		
	ConvertTimeValue: function (value) {
		// Given a value that represents bytes, return a string that repsents the value, rounded to the nearest storage unit:
	    var multipliers = [7*24*60*60*1000, 24*60*60*1000, 60*60*1000, 60*1000];
		var suffixes = [3, 2, 1, 0];
		var val = parseInt(value, 10);
		var newValue = val;
		var suffix = 0;
	
	    for (i=0; i < multipliers.length; i++) {
	        if (val >= multipliers[i]) {
	            newValue = val / multipliers[i];
	            suffix = suffixes[i];
	            break;
	        }
	    }
    
	    var time = new Object();
	    time.value = Math.round(newValue);
	    time.unit = suffix;
		return time;
	},

	/**************************************************************************
	* Gets time value in milliseconds.
	*
	* Parameter:
	*		value - settings value
	*		unit - settings value unit
	**************************************************************************/		
	GetTimeValue: function (value, unit) {
	  	var multipliers = [60*1000, 60*60*1000, 24*60*60*1000, 7*24*60*60*1000];
		return (value * multipliers[unit]);
	},
					 
	DateFormatter: function (value) {
		var NLS = dojo.i18n.getLocalization("nls", "general");	

		if ( value == 0 )
			return NLS.T_None;
		else
			return dojo.date.locale.format(new Date(value), {});
	},

	/**************************************************************************
	* Returns the <img> for the input status.
	*
	* Parameter:
	*		value - the status value
	**************************************************************************/		
	ConnectStatusFormatter: function (value) {
		var NLS = dojo.i18n.getLocalization("nls", "general");
		var result;
		switch(value) {
			case 0: 
				result =  "<img src='"+fbfwca_root+ "/jsp/CSS/images/ac16_normal_24.png' width='16' height='16' alt='" +
							NLS.T_OK + "' title='" + NLS.T_OK + "'/>"; 
				break;
			default : 
				result = "<img src='"+fbfwca_root+"/jsp/CSS/images/ac16_critical04_24.png' width='16' height='16' alt='" +
							NLS.T_Critical + "' title='" + NLS.T_Critical + "'/>";
		}
		return result;

	},
	
	/**************************************************************************
	* Returns the <img> for the input status.
	*
	* Parameter:
	*		value - the status value
	**************************************************************************/		
	StatusFormatter: function (value) {
		var NLS = dojo.i18n.getLocalization("nls", "general");
		var result;
		switch(value) {
			case 0: 
				result =  "<img src='"+fbfwca_root+"/jsp/CSS/images/ac16_normal_24.png' width='16' height='16' alt='" +
							NLS.T_OK + "' title='" + NLS.T_OK + "'/>"; 
				break;
			case 1: result =  "<img src='"+fbfwca_root+"/jsp/CSS/images/ac16_WarningMinor_24.png' width='16' height='16' alt='" +
							NLS.T_Warning + "' title='" + NLS.T_Warning + "'/>";
				break;
			case 2: result =  "<img src='"+fbfwca_root+"/jsp/CSS/images/ac16_critical04_24.png' width='16' height='16' alt='" +
							NLS.T_Critical + "' title='" + NLS.T_Critical + "'/>";
				break;
			case 3: result =  "<img src='"+fbfwca_root+"/jsp/CSS/images/ac16_fatal04_24.png' width='16' height='16' alt='" +
							NLS.T_Fatal + "' title='" + NLS.T_Fatal + "'/>";
				break;
			default : result = "<img src='"+fbfwca_root+"/jsp/CSS/images/ac16_unknown_24.png' width='16' height='16' alt='" +
							NLS.T_Unknown + "' title='" + NLS.T_Unknown + "'/>";
		}
		return result;

	},

	/**************************************************************************
	* Returns the short string for the input string if it's longer than
	* pre-determined max length.
	*
	* Parameter:
	*		value - the string value
	**************************************************************************/		
	TextFormatter: function (value) {
		var maxlength = 50;
		if ( value.length <= maxlength )
			return (value);
			
		var result;
		result = value.substring(0, 39) + "...";
		return result;

	},

	/**************************************************************************
	* Returns the short string for the input string if it's longer than
	* pre-determined max length.
	*
	* Parameter:
	*		value - the string value
	**************************************************************************/		
	LongTextFormatter: function (value) {
		var maxlength = 70;
		if ( value.length <= maxlength )
			return (value);
			
		var result;
		result = value.substring(0, 59) + "...";
		return result;

	},

	/**************************************************************************
	* Returns the short path notation for the input string.
	*
	* Parameter:
	*		value - the string value
	**************************************************************************/		
	PathFormatter: function (value) {
		var maxlength = 30;
		var endingMaxLen = 8;
		
		if ( value.length <= maxlength )
			return (value);
			
		var path = value;
		var idx = 0;
		
		if(value[value.length-1] == '\\')
		{
			path=value.substring(1,value.length-1);
		}
			
		if (path[0] == '\\') 
		{
			idx = 1;
		}
		if (path[1] == '\\')
		{
			idx = 2;
		}
		path = path.substring(idx, path.length);
	
		var tokens = path.split( "\\" );
		
		var lastToken = tokens[tokens.length-1];
		if (lastToken.length > endingMaxLen)
			lastToken = value.substring(value.length-endingMaxLen, value.length);
  		path = value.substring(0, idx);
  		result = path+tokens[0]+"\\"+tokens[1]+"\\..."+lastToken;
  		if (result.length > maxlength)
  			result = path+tokens[0]+"\\..."+lastToken;
		return result;

	},
		
	/**************************************************************************
	* Remove escape characters from input string.
	*
	* Parameter:
	*		value - the status value
	**************************************************************************/		
	RemoveEscape: function (value) {
		return unescape(value);

	}
};

}
