/*
	Copyright (c) 2004-2009, The Dojo Foundation All Rights Reserved.
	Available via Academic Free License >= 2.1 OR the modified BSD license.
	see: http://dojotoolkit.org/license for details
*/


if(!dojo._hasResource["fbfwca.Scripts"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["fbfwca.Scripts"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2009 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/
dojo.require("dijit.Toolbar");
dojo.require("dijit.form.DropDownButton");
dojo.require("dijit.Menu");
dojo.require("ibm.tivoli.tip.dijit.TIPMessageDialog");
dojo.require("ibm.tivoli.tip.dijit.TIPButton");		
dojo.require("fbfwca.Grid");
dojo.require("fbfwca.Common");
dojo.require("fbfwca.ajax");
dojo.require("fbfwca.Formatters");
dojo.requireLocalization("nls", "general", null, "");
dojo.requireLocalization("nls_messages", "messages", null, "");

dojo.provide("fbfwca.Scripts");

dojo.declare("fbfwca.Scripts", null, {

	/**************************************************************************
	* Constructor
	**************************************************************************/
	constructor : function() {
		console.log("fbfwca.Scripts(): in constructor");
		this.mainPageId = 'MainTableHolder';
		this.EditScriptId = 'EditScriptForm';
		this.connections = [];
		this.topics = [];
		this.gridObj = null;
		this.grid = null;
		this.folders = null;
		this.alerts = null;
		this.NLS = dojo.i18n.getLocalization("nls", "general");
		this.NLS_MESSAGES = dojo.i18n.getLocalization("nls_messages", "messages");
	},

  /**************************************************************************
  * switch to scripts sub page, if necessary
  **************************************************************************/
  initPage : function (Folders, Alerts) {
		this.folders = Folders;
		this.alerts = Alerts;
		this.InitScriptsTable(); 
  },

	/**************************************************************************
	* Cleanup function
	**************************************************************************/
	destroy : function () {
	  	// Destroy all subscriptions
	  	dojo.forEach(this.topics, dojo.unsubscribe);
	  	delete this.topics;
	  	
	  	// Destroy all event connections
		dojo.forEach(this.connections, dojo.disconnect);
	  	delete this.connections;

		if (dijit.byId(this.EditScriptId)) {
			console.log("destroy EditScriptForm");
			dijit.byId(this.EditScriptId).destroyRecursive();
		}
						  	
		delete this.mainPageId;
		delete this.EditScriptId;
		delete this.grid;
		delete this.gridObj;
		delete this.folders;
		delete this.scripts;  		
		delete this.NLS;
		delete this.NLS_MESSAGES;
 	},
	  	
	/**************************************************************************
	* Creates & loads the scripts table.
	*
	* Parameter:
	*		none
	**************************************************************************/		
	InitScriptsTable : function () {
    var textAlign;
    if(dojo.getComputedStyle(dojo.body()).direction == "rtl")
      textAlign = "text-align:right;";
    else
      textAlign = "text-align:left;";
      
		// Create the layout
		var layout = [
			{field: "_checked", name: "&nbsp;", width:"28px", editable:true, type:dojox.grid.cells.Bool,
					filter:false, styles: "text-align: center;" },
			{field: 'name', name: this.NLS.T_Name, width:'30%', cellClasses: "fauxLink", filter:true, styles:textAlign },
			{field: 'desc', name: this.NLS.T_Description, width:'70%', filter:true, 
					formatter:fbfwca.Formatters.LongTextFormatter, styles:textAlign }
		];

		// Create toolbar widgets
		var	createButton = new ibm.tivoli.tip.dijit.TIPButton(
				{	label: this.NLS.T_Create_a_Script,
					showLabel: false, 
					iconClass: "newIcon" 
				});
		this.connections.push (dojo.connect(createButton, 'onClick', dojo.hitch(this, this.PerformAction, "create")));

		var	deleteButton = new ibm.tivoli.tip.dijit.TIPButton(
				{	label: this.NLS.T_Delete_a_Script,
					showLabel: false, 
					iconClass: "deleteIcon" 
				});
		this.connections.push (dojo.connect(deleteButton, 'onClick', dojo.hitch(this, this.DeleteScript)));

		var menu = new dijit.Menu();
		this.connections.push (dojo.connect(menu, 'onOpen', dojo.hitch(this, this.OnClick_TableAction)));

		var item = new dijit.MenuItem({
				label: this.NLS.T_Create_a_Script,
				title: this.NLS.T_Create_a_Script,
         		iconClass: "newIcon",
				id: "CreateScriptID"
		});
		this.connections.push (dojo.connect(item, 'onClick', dojo.hitch(this, this.PerformAction, "create")));
		menu.addChild(item);

		item = new dijit.MenuItem({
				 label: this.NLS.T_Create_Like_Script,
				 title: this.NLS.T_Create_Like_Script,
				 iconClass: "newIcon",
				 id: "CreateLikeScriptID",
				 disabled: true
		});
		this.connections.push (dojo.connect(item, 'onClick', dojo.hitch(this, this.PerformAction, "createlike")));
		menu.addChild(item);
		
		item = new dijit.MenuItem({
				label: this.NLS.T_Modify_a_Script,
				title: this.NLS.T_Modify_a_Script,
         		iconClass: "editIcon",
				id: "ModifyScriptID",
				disabled: true
		});
		this.connections.push (dojo.connect(item, 'onClick', dojo.hitch(this, this.PerformAction, "modify")));
		menu.addChild(item);

		item = new dijit.MenuItem({
				label: this.NLS.T_Delete_a_Script,
				title: this.NLS.T_Delete_a_Script,
         		iconClass: "deleteIcon",
				id: "DeleteScriptID",
				disabled: true
		});
		this.connections.push (dojo.connect(item, 'onClick', dojo.hitch(this, this.DeleteScript)));
		menu.addChild(item);

		//menu.addChild(new dijit.MenuSeparator());
                       
		var actionsDropDown = new dijit.form.DropDownButton({
	        label: this.NLS.T_ACTIONS,
	        title: this.NLS.T_ACTIONS,
	        dropDown: menu
		});
  
		// Create the grid
		var gridParms = { storeURL : "scripts",
						layout : layout,
						//refreshButton : false,
						filterOn : false,
						createButton : createButton,
						deleteButton : deleteButton,
						actionsDropDown : actionsDropDown
					};
										
		this.gridObj = new fbfwca.Grid(gridParms, document.createElement('div'));
		dojo.byId("scriptsGridContainerNode").appendChild(this.gridObj.domNode);
		this.gridObj.resize();
		this.grid = this.gridObj.getTable();

		// Connect table default action
		this.connections.push (dojo.connect(this.grid, 'onCellClick', dojo.hitch(this, this.LaunchTableDefaultAction)));
		this.connections.push (dojo.connect(this.grid, "onCellMouseOver", this.ShowGridTooltip));
		this.connections.push (dojo.connect(this.grid, "onCellMouseOut", this.HideGridTooltip));       

	},

	/**************************************************************************
	* Resize table object
	**************************************************************************/		
	ResizeTable : function (){
		if (this.gridObj)
			this.gridObj.resize();
	},
	
	/**************************************************************************
	* Hide tooltip
	*
	* Parameter:
	*		e - event
	**************************************************************************/		
	HideGridTooltip : function (e){
		dijit.hideTooltip(e.cellNode);
		dijit._masterTT._onDeck=null;
	},

	/**************************************************************************
	* Shows tooltip
	*
	* Parameter:
	*		e - event
	**************************************************************************/		
	ShowGridTooltip : function (e) {
		var row;
		if (e.grid.getCell(e.cellIndex).field == "desc")
		{
			row = e.grid.getItem(e.rowIndex);
			dijit.showTooltip(e.grid.store.getValue(row, "desc", "0"), e.cellNode);
		}
	},
	
	/**************************************************************************
	* Handles default table action.
	*
	* Parameter:
	*		e - event
	**************************************************************************/		
	LaunchTableDefaultAction : function (e) {
		if (e.grid.getCell(e.cellIndex).field == "name")
		{
			this.PerformAction('modify');
			e.grid.selection.deselect(e.rowIndex);
		}
	},
		
	/**************************************************************************
	* Handles table action.
	*
	* Parameter:
	*		action - which action
	**************************************************************************/		
	PerformAction : function (action){
		if (action == 'create' )
			this.ShowScriptForm('create');
		else if (action == 'createlike' )
			this.ShowScriptForm('createlike');
		else if (action == 'modify' )
			this.ShowScriptForm('modify');
		else
			alert("Invalid action");
	},
	
	/**************************************************************************
	* Initializes and shows the Edit Script form.
	*
	* Parameter:
	*		action - which action
	**************************************************************************/		
	ShowScriptForm : function (action)
	{
		dijit.byId("EditScriptFormOK").attr("label", this.NLS.T_OK);
		dijit.byId("EditScriptFormCancel").attr("label", this.NLS.T_CANCEL);

		dijit.byId("EditScriptFormOK").attr("title", this.NLS.T_OK);
		dijit.byId("EditScriptFormCancel").attr("title", this.NLS.T_CANCEL);
				
		var div1 = document.getElementById("EditScriptFormHead");
		var name = dijit.byId('Script_name');
		var scriptData = null;

		if (action == "create" || action == "createlike")
		{
			div1.innerHTML = this.NLS.T_Create_a_Script;
			name.attr('disabled', false);
			name.promptMessage = this.NLS.T_REQUIRED_FIELD;
			this.ResetForm();
		}
		else
		{
			div1.innerHTML = this.NLS.T_Modify_a_Script;
			name.attr('disabled', true);
		}
		
		if ( action == "createlike" || action == "modify")
		{
				var item = this.grid.selection.getFirstSelected();
				var scriptname = this.grid.store.getValue(item, "name");
				if ( action == "modify" )
					name.attr('value', scriptname);
				
				scriptData = fbfwca.ajax.getScript(scriptname);
		}
    
    if (scriptData)
    {
			console.log("ShowScriptForm: restore from server");
			dijit.byId('Script_desc').attr('value', scriptData.desc);
			dijit.byId('Script_numclients').attr('value', scriptData.num_clients);
			var time = fbfwca.Formatters.ConvertTimeValue(scriptData.accept_timeout);
			dijit.byId('Script_acceptTimeout').attr('value', time.value);
			dijit.byId('Script_acceptTimeout_unitFS').attr('value', time.unit);
			delete time;
			
			time = fbfwca.Formatters.ConvertTimeValue(scriptData.complete_timeout);
			dijit.byId('Script_completeTimeout').attr('value', time.value);
			dijit.byId('Script_completeTimeout_unitFS').attr('value', time.unit);
					delete time;
			var i;
			for (i=0; i < scriptData.commands.length; i++) {
		        scriptData.commands[i] = scriptData.commands[i].replace(/\'/g,'\"');
			}
			dijit.byId('Script_commands').attr('value', scriptData.commands.join("\n"));
    }
    
		fbfwca.Common.ShowForm(this.EditScriptId, this.mainPageId);
	},

	/**************************************************************************
	* Resets widgets on the Edit Script form.
	* Mainly to initialize fields to default value.
	*
	* Parameter:
	*		none
	**************************************************************************/		
	ResetForm : function (){
		dijit.byId('Script_name').attr('value', '');
		dijit.byId('Script_desc').attr('value', '');
		
		dijit.byId('Script_numclients').attr('value', 10);
		dijit.byId('Script_acceptTimeout').attr('value', 15);
		dijit.byId('Script_acceptTimeout_unitFS').attr('value', '0');
		dijit.byId('Script_completeTimeout').attr('value', 15);
		dijit.byId('Script_completeTimeout_unitFS').attr('value', '0');
		dijit.byId('Script_AddCmdFS').attr('value', 0);
		dijit.byId('Script_commands').attr('value', '');

	},

	AddCommand : function (inputId) {
		// text area
		var obj = dojo.byId(inputId);
		var selectWidget = dijit.byId('Script_AddCmdFS');
		var selection = selectWidget.attr('value');

		// Add perform an initial scan command
    	if (selection == 1)
			obj.value+="<synchronize wait=0 />\n";
		
		// Add backup command
    	else if (selection == 2)
			obj.value+="<backup wait=0 />\n";
		
		// Add pause remote backup command
	   else if (selection == 3)
			obj.value+="<pause activate=1 />\n";
		
		// Add resume remote backup command
    	else if (selection == 4)
			obj.value+="<pause activate=0 />\n";
		
		// Add report command
		else if (selection == 5)
			obj.value+="<report />\n";
			
		// Reset selection box
		selectWidget.attr('value', 0);		
	},
	
/*
	OnClick_SetPauseResumeRB : function (e) {
		if (dijit.byId('Pause_resume_cb').checked == true )
		{
			fbfwca.Scripts.SetRBGroupDisabled(false);
		}
		else
		{
			fbfwca.Scripts.SetRBGroupDisabled(true);
		}
	},

	SetRBGroupDisabled : function (value)
	{
		dijit.byId('Unchanged_rb').attr('disabled', value);
		dijit.byId('PauseRemoteBk_rb').attr('disabled', value);
		dijit.byId('ResumeRemoteBk_rb').attr('disabled', value);
	},
*/

	/**************************************************************************
	* Issues a confirm message for the deletion.
	*
	* Parameter:
	*		none
	**************************************************************************/		
	DeleteScript : function () {
		var msgDlg;
		if ( this.grid.selection.getSelectedCount() == 0 )
		{
			msgDlg = new ibm.tivoli.tip.dijit.TIPMessageDialog(
					{message: this.NLS_MESSAGES.FBW1002E,
					 messageId: "FBW1002E",
					 type: "Error"});
		}
		else
		{
			var buttons = [this.NLS.T_OK, this.NLS.T_CANCEL];
			msgDlg = new ibm.tivoli.tip.dijit.TIPMessageDialog(
					{message: this.NLS_MESSAGES.FBW1001C,
					 messageId: "FBW1001C",
					 type: "Confirm",
					 callback: this.DeleteScriptCallback,
					 callbackParms: {grid: this.grid, gridObj: this.gridObj},
					 buttons: buttons});
		}
		msgDlg.show();

	},

	/**************************************************************************
	* Delete call back.  Called by TIPMessageDialog.
	*
	* Parameter:
	*		buttonPressedId - button pressed
	*		messageId - message ID
	*		checked - show checked box check
	*		parms - {grid: this.grid, gridObj: this.gridObj}
	**************************************************************************/		
	DeleteScriptCallback : function (buttonPressedId, messageId, checked, parms) {
		
		if (buttonPressedId == 0)
		{
		  var count = 0;
	    parms.grid.store.fetch(
		  {
			  query: {_checked:true},
			  onItem: function(item, request) {
					  console.log("DeleteScriptCallback: " + item.name);
					  fbfwca.ajax.deleteScript(item.name);
					  parms.grid.store.deleteItem(item);
					  count++;
			  }
		  });
		  parms.grid.store.save();
		  parms.grid.sort();
		  parms.gridObj.updateFooter(parms.grid.rowCount-count, 0);
		}
	},

	/**************************************************************************
	* Handles button pressed on Edit Script form.
	*
	* Parameter:
	*		buttonId - which button was pressed (0=OK, 1=Cancel)
	**************************************************************************/		
	OnClick_EditScript : function (buttonId) {

		// Cancel button
		if ( buttonId == 1 )
		{
			this.CleanupForm();
			fbfwca.Common.HideForm(this.EditScriptId, this.mainPageId);
			this.RefreshTables();
			return;
		}
			
		// OK button
		var name = dijit.byId('Script_name');
		
		// Make sure required fields were entered.
		if (name.attr('disabled') == false && !name.validate(true) )
			return;
		
		var script = new Object();
		script.name = name.attr('value');
		script.desc = dijit.byId('Script_desc').attr('value');
		script.num_clients = dijit.byId('Script_numclients').attr('value');
		script.accept_timeout = fbfwca.Formatters.GetTimeValue(dijit.byId('Script_acceptTimeout').attr('value'),
										 		dijit.byId('Script_acceptTimeout_unitFS').attr('value'));
		script.complete_timeout = fbfwca.Formatters.GetTimeValue(dijit.byId('Script_completeTimeout').attr('value'),
										 		dijit.byId('Script_completeTimeout_unitFS').attr('value'));

		script.commands = dijit.byId('Script_commands').attr('value').split("\n");
		var i;
		for (i=0; i < script.commands.length; i++) {
            script.commands[i] = script.commands[i].replace(/\"/g,'\'');
        }
		
		// Create action
		var response;
		if (name.attr('disabled') == false)
			response = fbfwca.ajax.createScript(script);
		
		// Modify action
		else
			response = fbfwca.ajax.modifyScript(script);
	
		delete script.commands;
		delete script;
		
		if ( response == -1 )
			return;

		this.CleanupForm();
		fbfwca.Common.HideForm(this.EditScriptId, this.mainPageId);
		this.RefreshTables();
	},

	RefreshTables : function () 
	{
		this.gridObj.onRefresh();
		this.folders.gridObj.onRefresh();
		this.alerts.gridObj.onRefresh();
	},
	
	/**************************************************************************
	* Enables or disables actions based on user's action selection.
	*
	* Parameter:
	*		none
	**************************************************************************/		
	OnClick_TableAction : function () {
		var count = this.grid.selection.getSelectedCount();
		
		var CreateAction = dijit.byId("CreateLikeScriptID");
		var DeleteAction = dijit.byId("DeleteScriptID");
		var ModifyAction = dijit.byId("ModifyScriptID");
		
		if ( count == 0 )
		{
			CreateAction.attr('disabled', true);
			DeleteAction.attr('disabled', true);
			ModifyAction.attr('disabled', true);
		}
		else if ( count == 1 )
		{
			CreateAction.attr('disabled', false);
			DeleteAction.attr('disabled', false);
			ModifyAction.attr('disabled', false);
		}
		else
		{
			CreateAction.attr('disabled', true);
			DeleteAction.attr('disabled', false);
			ModifyAction.attr('disabled', true);
		}
	},
	
	/**************************************************************************
	* Performs cleanup for the script form.
	* Important to reset any required widgets & hide tooltip
	*
	* Parameter:
	*		none
	**************************************************************************/		
	CleanupForm : function () {
		var widget = dijit.byId('Script_name');
		if (widget.attr('disabled') == false)
		{
			dijit.hideTooltip(widget.domNode);
			widget.reset();
		}
	}

});

}
