/*
	Copyright (c) 2004-2009, The Dojo Foundation All Rights Reserved.
	Available via Academic Free License >= 2.1 OR the modified BSD license.
	see: http://dojotoolkit.org/license for details
*/


/*
	Copyright (c) 2004-2009, The Dojo Foundation All Rights Reserved.
	Available via Academic Free License >= 2.1 OR the modified BSD license.
	see: http://dojotoolkit.org/license for details
*/


if(!dojo._hasResource["ibm.tivoli.tip.dijit.DndListbox"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.DndListbox"] = true;
/**
 * A list box supporting drag and drop, filtering and sorting
 * This is different that a select list in that the DndListbox's value is its contents, not what is selected
 * 
 * Creating a DndListbox using markup
 * <div id="l1" dojoType="ibm.tivoli.tip.dijit.DndListbox" filtered="true" sorted="true" accept="foo" type="foo"></div>
 * 		filtered: if true, the filter input box is displayed and the user may filter the list's contents
 * 		sorted: if true, the list is kept sorted. Dragging and dropping within the list is prohibited
 * 		accept: a comma separated list of types that may be dropped into this list box
 * 		type: a comma separated list of the types of items in this list
 * 
 * Creating a DndListbox programmatically
 * 	var zappas = new ibm.tivoli.tip.dijit.DndListbox({filtered:false, sorted:true, accept:["zappa"], type:[""celebrity", "zappa"]}, domNode);
 * 	zappas.startup();
 * 	zappas.attr('value', [ 'Frank', 'Dweezil', 'Moon Unit', 'Ahmet', 'Diva Thin Muffin Pigeen', 'Gail']);
 * 
 * Events:
 * 	OnChange: fired after something is dropped in the list
 * 					Is also fired while the list is being filtered, which can happen quite often while the 
 * 					user is typing, so be careful what you do in the event handler
 * 
 * TODO: the template includes some fixed widths that should be removed
 * TODO: support dojo.data.*Store as a data source
 * TODO: TIP compliant styling
 * TODO: support a disabled attribute (prevent user interaction and form submission)
 * TODO: Accessibility support
 * TODO: improve the creator function so it doesn't morph dropped items into this list's type?
 */
dojo.provide("ibm.tivoli.tip.dijit.DndListbox");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.dnd.Container");
dojo.require("dojo.dnd.Manager");
dojo.require("dojo.dnd.Source");
dojo.require("dojo.i18n");
dojo.require("ibm.tivoli.tip.dojo.AccentMapper");

dojo.declare("ibm.tivoli.tip.dijit.DndListbox", [dijit._Widget, dijit._Templated, ibm.tivoli.tip.dojo.AccentMapper],
{
	widgetsInTemplate: true,
	templateString:"<div class=\"tip-DndListbox\" >\r\n\t<label for=\"${id}_listbox\">${label}</label>\r\n\t<div class=\"quick-filter\" dojoAttachPoint=\"_filterBox\" >\r\n\t\t<table border=\"0\" cellspacing=\"0\" cellpadding=\"0\" style=\"width:100%\">\r\n\t\t<tr>\r\n\t\t\t<td class=\"qf-lcap\" >&#160;</td>\r\n\t\t\t<td class=\"qf-mid\" valign=\"middle\">\r\n\t\t\t\t<div style=\"position:relative;\">\r\n\t\t\t\t\t<label title=\"${filterTip}\" >\r\n\t\t\t\t\t\t<input dojoAttachPoint=\"_filterInput\" type=\"text\" class=\"qf-textfield\" tabindex=\"0\"\r\n\t\t\t\t\t\t\tdojoAttachEvent=\"onblur:quickFilterDefault, onfocus:quickFilterFocus, onkeyup:onFilterChange\" \r\n\t\t\t\t\t\t/>\r\n\t\t\t\t\t</label>\r\n\t\t\t\t\t<span dojoAttachPoint=\"_filterHint\" class=\"qf-hint\" dojoAttachEvent=\"onclick:quickFilterClickHint\">${filterHint}</span>\r\n\t\t\t\t</div>\r\n\t\t\t</td>\r\n\t\t\t<td class=\"qf-rcap\" dojoAttachEvent=\"onclick:stopFilter,onmouseover:quickFilterClearOver,onmouseout:quickFilterClearOut\"><div class=\"qf-clear\">x</div></td>\r\n\t\t</tr>\r\n\t\t</table>\r\n\t</div>\r\n\t<div id=\"${id}_listbox\" class=\"balContainer\" dojoAttachPoint=\"_scrollBox\" tabindex=\"-1\">\r\n\t\t<ol class=\"dndContainer\" dojoAttachPoint=\"_sourceContainer\"></ol>\r\n\t</div>\r\n</div>\r\n",
	filtered: true,	// show the filter text box?
	sorted: false,		// sort the list contents?
	accept: ["text"],
	type: ["text"],
	name: "",
	label: "",
	_source: null,		// my dojo.dnd.Source
	_filterTimer: -1,	// timer ID
	// nls strings
	filterTip: "",
	filterHint: "",
	
	constructor: function()
	{
		dojo.requireLocalization("ibm.tivoli.tip.dijit", "resources", null, "ROOT,ar,cs,da,de,en,en-us,es,fi,fr,he,hr,hu,it,ja,ko,nl,no,pl,pt-br,ro,ru,sl,sv,tr,zh-cn,zh-tw", "ROOT,ar,cs,da,de,en,en-us,es,fi,fr,he,hr,hu,it,ja,ko,nl,no,pl,pt-br,ro,ru,sl,sv,tr,zh-cn,zh-tw");
		// preload imagery
		if(dojo.hasClass(dojo.body(), "tundra")) {
			var imgpath = dojo.moduleUrl("ibm.tivoli.tip.dijit", "themes/images/");
			var m = new Image(); m.src = imgpath + "dndMove.png";
			var n = new Image(); n.src = imgpath + "dndNoMove.png";
			var o = new Image(); o.src = imgpath + "lb_qf_bkg_cap_l_focus.gif";
			var p = new Image(); p.src = imgpath + "lb_qf_bkg_mid_focus.gif";
			var q = new Image(); q.src = imgpath + "lb_qf_bkg_cap_r_focus.gif";
		}
	},
	postMixInProperties: function()
	{
		this.filterTip = dojo.i18n.getLocalization("ibm.tivoli.tip.dijit", "resources").FILTERTIP;
		this.filterHint = dojo.i18n.getLocalization("ibm.tivoli.tip.dijit", "resources").FILTERHINT;
		this.inherited(arguments);
	},
	startup: function()
	{
		this._source = new dojo.dnd.Source(this._sourceContainer, {creator: dojo.hitch(this, this._creator), accept: this.accept});
		
		if(this.sorted) {
			// prevent dragging items w/in the list
			this._source.checkAcceptance = dojo.hitch(this._source, function(source, nodes) {
				if(source === this) 
					return false;
				
				// a copy of dojo.dnd.Source.checkAcceptance
				for(var i = 0; i < nodes.length; ++i){
					var type = source.getItem(nodes[i].id).type;
					// type instanceof Array
					var flag = false;
					for(var j = 0; j < type.length; ++j){
						if(type[j] in this.accept){
							flag = true;
							break;
						}
					}
					if(!flag){
						return false;	// Boolean
					}
				}
				return true;
			});
		}
		
		// keep the list updated after dropping
		this.connect(this._source, "onDrop", this._onDrop);
		
		if(!this.filtered) {
			dojo.style(this._filterBox, "display", "none");
		}
		
		// event handlers to manage focus and keyboard input
		this.connect(this._scrollBox, "onkeypress", this._onListKeyHandler);
		this.connect(this._scrollBox, "onclick", this._findFocusedItem);
		
		this.inherited(arguments);
	},
	destroy: function()
	{
		if(this._fhandle) 
			dojo.disconnect(this._fhandle);
		this.inherited(arguments);
	},
	/**
	 * addData()
	 * 
	 * Add data to the available list
	 * 
	 * Input:
	 * 	data: the data being added to the available list
	 * 
	 */
	addData: function(data)
	{
		this._source.insertNodes(false, dojo.isArray(data) ? data : [data]);
		this._onListChange();
	},
	deleteSelected: function()
	{
		this._source.deleteSelectedNodes();
		this._onListChange();
	},
	deleteAll: function()
	{
		this._source.selectAll().deleteSelectedNodes();
		this._onListChange();
	},
	// attribute getters/setters
	_getValueAttr: function()
	{
		var itemArray = [];
		dojo.query("input", this._sourceContainer).forEach(function(n) {
			itemArray.push(n.value);
		});
		return itemArray;
	},
	_setValueAttr: function(value)
	{
		// TODO: setting the value should replace the list contents, not append
		this.addData(value);
	},
	getItem: function(key)
	{
		return this._source.getItem(key);
	},
	delItem: function(key)
	{
		this._source.delItem(key);
		var n = dojo.byId(key);
		dojo.destroy(n);
	},
	getAllNodes: function()
	{
		return this._source.getAllNodes();
	},
	getSelectedNodes: function()
	{
		if(this._source == null)
			return [];
		else
			return this._source.getSelectedNodes();
	},
	deleteSelectedNodes: function()
	{
		return this._source.deleteSelectedNodes();
	},
	selectAll: function()
	{
		this._source.selectAll();
	},
	selectNone: function()
	{
		this._source.selectNone();
	},
	insertNodes: function(nodes)
	{
		this._source.insertNodes(false, nodes);
		this._onListChange();
	},
	_creator: function(item, hint)
	{
		// this function isn't quite right. when you drop an object into this list
		// it morphs into this.type, and forgets what it was.
		var n;
		if("avatar" === hint) {
			n = dojo.create("span", {innerHTML: item.toString()});
		}
		else {
			var val = item.toString();
			n = dojo.create("li", {dndData: val, tabindex: -1});
			var ii = dojo.create("input", {type:"text", readonly:"true", value: val, tabindex: -1});
			if(this.name) {
				dojo.attr(ii, {name: this.name});
			}
			n.appendChild(ii);
		}
		//console.log("item: ", item, "type: ",this.type);
		return {node: n, data: item, type: this.type};
	},
	//----------------------- manage focus ----------------------------------------
	focusedItem: null,
	/**
	 * called onclick in the _scrollBox
	 * find the clicked-on item, and give it focus
	 */
	_findFocusedItem: function(evt)
	{
		console.log("DndListbox._findFocusedItem");
		var nl = this.getAllNodes();
		if(nl.length == 0)
			return;
		
		var elem = evt.target;
		while(elem != null && "string" == typeof elem.tagName && elem.tagName.toLowerCase() != "li") {
			elem = elem.parentNode;
		}
		if(null != elem && "string" == typeof elem.tagName && elem.tagName.toLowerCase() == "li") {
			this._moveFocus(elem);
		}
	},
	_onListFocus: function(evt)
	{
		console.log("DndListbox._onListFocus: ", evt.target);
		console.log("   _onListFocus focusedItem before: ", this.focusedItem);
		var new_focused_item = this._findFirstItem();
		this._moveFocus(new_focused_item);
		console.log("   _onListFocus focusedItem after: ", this.focusedItem);
	},
	_onListKeyHandler: function(evt)
	{
		console.log("DndListBox._onListKeyHandler [%s: %s] key='%s'", evt.target.tagName, evt.target.id, evt.charOrCode);
		
		var new_focused_item = null;
		switch(evt.charOrCode) {
		case dojo.keys.TAB:
			// the list looses focus
			console.log("--- TAB");
			this._moveFocus(null);
			// must be done async do focus can move to the next element before
			// the top list item gets tabindex=0, or shift-tab always lands there
			window.setTimeout(dojo.hitch(this, this._swizzleTabindex), 10);
			break;
		case dojo.keys.DOWN_ARROW:
			// move focus down one list item
			console.log("--- DOWN");
			new_focused_item = this._findNextItem();
			this._moveFocus(new_focused_item);
			dojo.stopEvent(evt);
			break;
		case dojo.keys.UP_ARROW:
			// move focus up one list item
			console.log("--- UP");
			new_focused_item = this._findPrevItem();
			this._moveFocus(new_focused_item);
			dojo.stopEvent(evt);
			break;
		case " ":
			// toggle focused item selection
			console.log("--- SPACE");
			if(null !== this.focusedItem) {
				var isSelected = this._isSelected(this.focusedItem);
				if(!evt.ctrlKey && !evt.shiftKey) {
					this._source.selectNone();
				}
				if(isSelected) {
					dojo.removeClass(this.focusedItem, "dojoDndItemSelected");
					dojo.removeClass(this.focusedItem, "dojoDndItemAnchor");
				}
				else {
					if(this._source.getSelectedNodes().length == 0) {
						this._source.anchor = this.focusedItem;
						dojo.addClass(this.focusedItem, "dojoDndItemAnchor");
					}
					else {
						dojo.addClass(this.focusedItem, "dojoDndItemSelected");
					}
				}
				this._source.selection[this.focusedItem.id] = this._isSelected(this.focusedItem) ? 1 : 0;
				this.OnChange();
			}
			dojo.stopEvent(evt);
			break;
		}
	},
	_isSelected: function(node)
	{
		return dojo.hasClass(this.focusedItem, "dojoDndItemSelected") 
			|| dojo.hasClass(this.focusedItem, "dojoDndItemAnchor");
	},
	_findFirstItem: function()
	{
		var nl = this._source.getAllNodes();
		return nl.length > 0 ? nl[0] : null;
	},
	_findNextItem: function()
	{
		console.log("DndListbox._findNextItem");
		console.log("   focusedItem: ", this.focusedItem);
		
		var nodes = this._source.getAllNodes();
		if(nodes.length === 0)
			return null;
		
		var i = 0;
		if(null !== this.focusedItem) {
			for(i = 0; i < nodes.length; ++i) {
				if(nodes[i] === this.focusedItem) {
					++i;
					break;
				}
			}
		}
		console.log("   next item: ", i);
		return (i < nodes.length) ? nodes[i] : nodes[nodes.length-1];
	},
	_findPrevItem: function()
	{
		console.log("DndListbox._findPrevItem");
		console.log("   focusedItem: ", this.focusedItem);
		var nodes = this._source.getAllNodes();
		if(nodes.length === 0)
			return null;
		
		var i = nodes.length -1;
		for(;i > 0; --i) {
			if(nodes[i] === this.focusedItem) {
				--i;
				break;
			}
		}
		console.log("   prev item: ", i);
		return (i >= 0) ? nodes[i] : nodes[0];
	},
	_moveFocus: function(new_focused_item)
	{
		console.log("_moveFocus from ", this.focusedItem, " to ", new_focused_item);
		if(this.focusedItem !== new_focused_item) {
			if(this.focusedItem) {
				dojo.removeClass(this.focusedItem, "dojoDndItemFocused");
				dojo.attr(this.focusedItem, "tabindex", "-1");
			}
			if(null !== new_focused_item) {
				console.log("   focusing ", new_focused_item.id);
				dojo.addClass(new_focused_item, "dojoDndItemFocused");
				dojo.attr(new_focused_item, "tabindex", "0");
				new_focused_item.focus();
			}
			this.focusedItem = new_focused_item;
		}
		console.log("  _moveFocus  focusedItem after: ", this.focusedItem);
	},
	_onDrop: function(source, nodes, copy)
	{
		console.log("DndListbox._onDrop");
		this._onListChange();
	},
	_fhandle: null,
	_swizzleTabindex: function()
	{
		var i;
		var nl = this._source.getAllNodes();
		if(nl.length > 0) {
			if(this._fhandle) dojo.disconnect(this._fhandle);
			if(this._bhandle) dojo.disconnect(this._bhandle);
			this._fhandle = dojo.connect(nl[0], "onfocus", this, this._onListFocus);
			dojo.attr(nl[0], "tabindex", "0");
		}
		for(i = 1; i < nl.length; ++i) {
			dojo.attr(nl[i], "tabindex", "-1");
		}
	},
	_onListChange: function()
	{
		console.log(this.id + "._onListChange");
		if(this.filtered) {
			// unfilter
			this.stopFilter();
		}

		if(this.sorted) {
			// sort
			var list = this._source;
			var itemArray = [];
			this._source.forInItems(function(obj, id, container) {
				itemArray.push(obj.data);
				list.delItem(id);
				list.node.removeChild(dojo.byId(id));
			}, null);
			list = null;
			itemArray.sort(function(a, b) {return a.toString().localeCompare(b.toString());});
			this._source.insertNodes(false, itemArray);
		}
		this._source.sync();
		this._swizzleTabindex();
		if(this.focusedItem) {
			this._moveFocus(this._findFirstItem());
		}
		this.OnChange();	// fire 
	},
	// -------------------- filtering support -------------------------------
	quickFilterFocus: function(evt) 
	{
		dojo.addClass(this._filterBox, "quick-filter-focus");
		this.quickFilterHint();
	},
	quickFilterDefault: function (evt) 
	{
		dojo.removeClass(this._filterBox, "quick-filter-focus");
		dojo.removeClass(this._filterBox, "quick-filter-over");
		this.quickFilterHint();
	},

	/**
	* Handle the keyUp event on an input element to start a 
	* filter operation on the list
	*/
	onFilterChange: function(evt)
	{
		var inputElement = evt.target;
		this.startFilter();
	},
	/**
	 * Give the filter text box focus when the hint test is clicked
	 */
	quickFilterClickHint: function()
	{
		this._filterInput.focus();
	},
	/**
	 * Manage visibility of the filter hint
	 */
	quickFilterHint: function()
	{
		if(this._filterInput.value.length > 0) {
			dojo.addClass(this._filterHint, "hidden");
		}
		else {
			dojo.removeClass(this._filterHint, "hidden");
		}
	},
	quickFilterClearOver: function(evt)
	{
		dojo.addClass(dojo.query("div.qf-clear", evt.target.parentNode)[0], "hover");
	},
	quickFilterClearOut: function(evt)
	{
		dojo.removeClass(dojo.query("div.qf-clear", evt.target.parentNode)[0], "hover");
	},
	/**
	* Set a timer that will filter the list when it expires.
	*/
	startFilter: function()
	{
		window.clearTimeout( this._filterTimer );
		this._filterTimer = window.setTimeout( dojo.hitch(this, this.filter), 400 );
		return;
	},
	/**
	* Stops the background filter operation by clearing the window
	* timer. 
	*/
	stopFilter: function()
	{
		// stop the timer
		window.clearTimeout( this._filterTimer );
		this._filterTimer = -1;

		// clear the filter if necessary
		if(this._filterInput.value.length > 0) {
			this._filterInput.value = "";
			this.filter();
		}
		return;
	},
	/**
	 * Filter the list box so only those entries matching the what the user has
	 * typed in the filter box are shown
	 */
	filter: function()
	{
		//	Extract the current source from the input element
		var filter_val = this.mapAccents(dojo.trim(this._filterInput.value));
		var filter_re = new RegExp(filter_val, "i");

		var that = this;
		dojo.query("input", this._sourceContainer).forEach(function(n){
			var dndNode = n.parentNode;
			if(filter_re.test(that.mapAccents(n.value))) {
				dojo.removeClass( dndNode, "qfHide");
				dojo.addClass( dndNode, "dojoDndItem" );
			}
			else
			{
				dojo.removeClass( dndNode, "dojoDndItem" );
				dojo.addClass( dndNode, "qfHide" );
			}
		});
		that = null;
		this._source.sync();
		return;
	},
 
	// ------------------------ external events --------------------
	OnChange: function()
	{
	}
});

}
